@@ -1,8 +1,7 @@
 ;;; Dezyne --- Dezyne command line tools
 ;;;
-;;; Copyright © 2021 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
+;;; Copyright © 2021, 2022 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
 ;;; Copyright © 2021 Rutger van Beusekom <rutger@dezyne.org>
-;;; Copyright © 2023 Paul Hoogendijk <paul@dezyne.org>
 ;;;
 ;;; This file is part of Dezyne.
 ;;;
@@ -26,13 +25,13 @@
 (define-module (dzn code-util)
   #:use-module (srfi srfi-1)
   #:use-module (srfi srfi-26)
-  #:use-module ((oop goops) #:renamer (lambda (x) (if (member x '(<port> <foreign>)) (symbol-append 'goops: x) x)))
-  #:use-module (dzn goops)
+
+  #:use-module (dzn ast goops)
   #:use-module (dzn ast)
-  #:use-module (dzn command-line)
   #:use-module (dzn indent)
   #:use-module (dzn misc)
   #:use-module (dzn shell-util)
+
   #:export (code-util:dump
             code-util:file-name
             code-util:foreign-conflict?
@@ -55,30 +54,14 @@
   (let ((base (basename (ast:source-file root) ".dzn")))
     (code-util:file-name base dir ext)))
 
-(define (code-util:handle-exceptions file-name)
-  (lambda (key . args)
-    (case key
-      ((system-error)
-       (let ((errno (system-error-errno (cons key args))))
-         (format (current-error-port) "~a: ~a\n"
-                 (strerror errno) file-name))
-       (exit EXIT_FAILURE))
-      (else (format (current-error-port) "internal error: ~a: ~a: ~s\n"
-                    file-name key args)
-            (exit EXIT_FAILURE)))))
-
 (define* (code-util:dump root generate #:key file-name)
-  (let ((debug? (dzn:command-line:get 'debug #f)))
-    (cond
-      ((equal? file-name "-")
-        (generate))
-      (else
-        (catch (if debug? 'none #t)
-          (lambda ()
-            (mkdir-p (dirname file-name))
-            (with-output-to-file file-name
-              generate))
-          (code-util:handle-exceptions file-name))))))
+  (cond
+   ((equal? file-name "-")
+    (generate))
+   (else
+    (mkdir-p (dirname file-name))
+    (with-output-to-file file-name
+      generate))))
 
 (define-method (code-util:base-name (o <foreign>))
   (string-join (ast:full-name o) "_"))
@@ -97,5 +80,5 @@
 
 (define-method (code-util:generate-source? (o <root>))
   (find (conjoin (negate ast:imported?)
-                 (disjoin (is? <component>) (is? <system>)))
+                 (disjoin (is? <component>) (is? <foreign>) (is? <system>)))
         (ast:model* o)))
