@@ -1,6 +1,6 @@
 ;;; Dezyne --- Dezyne command line tools
 ;;;
-;;; Copyright © 2015, 2016, 2017, 2019, 2020, 2021, 2022, 2023 Janneke Nieuwenhuizen <janneke@gnu.org>
+;;; Copyright © 2015, 2016, 2017, 2019, 2020, 2021, 2022, 2023 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
 ;;; Copyright © 2015, 2017, 2021 Rutger van Beusekom <rutger@dezyne.org>
 ;;; Copyright © 2017, 2019 Rob Wieringa <rma.wieringa@gmail.com>
 ;;; Copyright © 2018 Filip Toman <filip.toman@verum.com>
@@ -25,19 +25,18 @@
 ;;; Code:
 
 (define-module (dzn code c)
-  #:use-module ((oop goops) #:renamer (lambda (x) (if (member x '(<port> <foreign>)) (symbol-append 'goops: x) x)))
-  #:use-module (ice-9 match)
-  #:use-module (ice-9 pretty-print)
-
   #:use-module (srfi srfi-1)
   #:use-module (srfi srfi-26)
 
+  #:use-module (ice-9 match)
+  #:use-module (ice-9 pretty-print)
+
+  #:use-module (dzn ast goops)
   #:use-module (dzn ast)
+  #:use-module (dzn code dzn)
   #:use-module (dzn code)
   #:use-module (dzn code-util)
-  #:use-module (dzn code dzn)
   #:use-module (dzn config)
-  #:use-module (dzn goops)
   #:use-module (dzn misc)
   #:use-module (dzn templates))
 
@@ -63,7 +62,7 @@
               ((compose string-upcase code:file-name) o)))
 
 (define-method (c:get-trigger-port-type (o <trigger>))
-  (c:name (parent (.event o) <interface>)))
+  (c:name (ast:parent (.event o) <interface>)))
 
 (define-method (c:comma (o <list>))
   (if (null? o) "" ","))
@@ -88,7 +87,7 @@
   (let ((type (.type o)))
     (match type
      (($ <enum>) "uint8_t")
-     (($ <int>) (c:range-type (.type o)))
+     (($ <subint>) (c:range-type (.type o)))
      (($ <extern>) (list ((compose .value .value .type) o)))
      (_ (ast:name type)))))
 
@@ -112,8 +111,11 @@
 (define-method (c:name (o <model>))
   (ast:full-name o))
 
+(define-method (c:model-parent-name (o <model>))
+  (c:name o))
+
 (define-method (c:model-parent-name (o <ast>))
-  (c:name (parent o <model>)))
+  (c:model-parent-name (ast:parent o <model>)))
 
 ;; enum stuff
 (define-method (c:enum-complete-name-upcase (o <enum>))
@@ -123,9 +125,19 @@
   (map (string->enum-field o) (ast:field* o) (iota (length (ast:field* o)))))
 
 ;;enum main stuff
+(define-method (c:get-all-enums (o <root>))
+  (tree-collect (is? <enum>) o))
+
 (define-method (c:get-all-enums (o <ast>))
-  (let ((root (parent o <root>)))
-    (tree-collect (is? <enum>) root)))
+  (let ((root (ast:parent o <root>)))
+    (c:get-all-enums root)))
+
+(define-method (c:get-all-local-enums (o <root>))
+  (filter (negate ast:imported?) (c:get-all-enums o)))
+
+(define-method (c:get-all-local-enums (o <ast>))
+  (let ((root (ast:parent o <root>)))
+    (c:get-all-local-enums root)))
 
 (define-method (c:enum-printed-name (o <enum-field>))
   (ast:name (.type o)))
@@ -149,8 +161,8 @@
 
 ;; helper struct stuff
 (define-method (c:equal? (a <type>) (b <type>))
-  (and(is-a? a <enum>)
-      (is-a? b <enum>)))
+  (and (is-a? a <enum>)
+       (is-a? b <enum>)))
 
 ;; closure struct in triggers of all the components
 (define-method (c:get-incoming-triggers-from-model (o <root>))
@@ -204,13 +216,13 @@
   (let ((type (ast:type o)))
     (match type
       (($ <enum>) "uint8_t")
-      (($ <int>) (c:range-type (ast:type o)))
+      (($ <subint>) (c:range-type (ast:type o)))
       (($ <extern>) (if (string= "int" ((compose ast:name .type) o))
                         "int16_t"
                         ((compose .value .type) o)))
       (_ (code:type-name o)))))
 
-(define-method (c:range-type (o <int>))
+(define-method (c:range-type (o <subint>))
   (let*((range (.range o))
         (from ( .from range))
         (to (.to range)))
@@ -248,6 +260,24 @@
                  (is? <foreign>))
         (ast:model* o)))
 
+(define-method (c:generate-source? (o <root>))
+  (find (conjoin (negate ast:imported?)
+                 (disjoin (is? <component>)
+                          (is? <enum>)
+                          (conjoin (is? <interface>)
+                                   (compose
+                                    pair?
+                                    code:enum-definer))
+                          (is? <system>)))
+        (ast:model* o)))
+
+(define-method (c:connect-instance-name-right (o <binding>))
+  (let* ((end-point (.right o))
+         (instance (.instance end-point))
+         (name (.name instance)))
+    (if (not (is-a? (.type instance) <foreign>)) name
+        (string-append name ".base"))))
+
 
 ;;;
 ;;; Entry points.
@@ -258,12 +288,12 @@
 
   (code-util:foreign-conflict? root)
 
-  (let ((root (code:normalize+determinism root)))
+  (let ((root (code:om+determinism root)))
     (let ((generator (code-util:indenter (cute x:header root)))
           (file-name (code-util:root-file-name root dir ".h")))
       (code-util:dump root generator #:file-name file-name))
 
-    (when (or (code-util:generate-source? root)
+    (when (or (c:generate-source? root)
               (c:generate-foreign-skel? root))
       (let ((generator (code-util:indenter (cute x:source root)))
             (file-name (code-util:root-file-name root dir ".c")))
